/**
* @license
* Copyright © 2015 Intuilab
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Intuilab shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Intuilab.
*/

/**
* Inheritance on EventEmitter base class
* @type {EventEmitter}
*/
Chronometer.prototype = new EventEmitter();        // Here's where the inheritance occurs
Chronometer.prototype.constructor = Chronometer;

/**
* @constructor
*/
function Chronometer() {
    this.reset();
}

/**
* Launch the chronometer
*/
Chronometer.prototype.start = function () {
    if (this._timer == null) {
        
        var self = this;
        this._timer = setInterval(function () { self.update() }, 1000);
        this.emit('started');
    }
};
/**
* Pause the chronometer
*/
Chronometer.prototype.pause = function () {
    if (this._timer != null) {
        clearInterval(this._timer);
        this._timer = null;

        this.emit('paused');
    }
};

/**
* Reset the chronometer
*/
Chronometer.prototype.reset = function () {
    this.setSeconds(0);
    this.setMinutes(0);
    this.setHours(0);
    this.formatTime();
};

/**
 * Reset the chronometer
 */
Chronometer.prototype.setTime = function (formatedTime) {

    var res = formatedTime.split(":",3);

    if ((res != null) && (res.length <= 3)){
        var s = parseInt(res[2]);
        var m = parseInt(res[1]);
        var h = parseInt(res[0]);

        this.setSeconds(s);
        this.setMinutes(m);
        this.setHours(h);
        this.formatTime();
    }

};

/**
* Update the chronometer
*/
Chronometer.prototype.update = function () {

    // increase seconds
    if (this.seconds < 59) {
        this.setSeconds(this.seconds + 1);
    } else {
        this.setSeconds(0);

        // increase minutes
        if (this.minutes < 59) {
            this.setMinutes(this.minutes + 1);
        } else {
            this.setMinutes(0);

            // increase hours
            this.setHours(this.hours + 1);
        }
    }

    // always format time
    this.formatTime();
};
/**
* Format current time
*/
Chronometer.prototype.formatTime = function () {
    this.time =  this.formattedHours + ":" + this.formattedMinutes + ":" + this.formattedSeconds;
    this.emit('timeChanged', [this.hours, this.minutes, this.seconds]);
};

/**
* Set seconds property
*/
Chronometer.prototype.setSeconds = function (seconds) {
    if (this.seconds != seconds) {
        this.seconds = seconds;
        this.emit('secondsChanged');

        this.formattedSeconds = (this.seconds < 10 ? "0" + this.seconds : this.seconds);
        this.emit('formattedSecondsChanged');
    }
};

/**
* Set minutes property
*/
Chronometer.prototype.setMinutes = function (minutes) {
    if (this.minutes != minutes) {
        this.minutes = minutes;
        this.emit('minutesChanged');

        this.formattedMinutes = (this.minutes < 10 ? "0" + this.minutes : this.minutes);
        this.emit('formattedMinutesChanged');
    }
};

/**
* Set hours property
*/
Chronometer.prototype.setHours = function (hours) {
    if (this.hours != hours) {
        this.hours = hours;
        this.emit('hoursChanged');

        this.formattedHours = (this.hours < 10 ? "0" + this.hours : this.hours);
        this.emit('formattedHoursChanged');
    }
};