/**
 * @license
 * Copyright � 2014 Intuilab
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability,
 * fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability,
 * whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
 *
 * Except as contained in this notice, the name of Intuilab shall not be used in advertising or otherwise to promote the sale,
 * use or other dealings in this Software without prior written authorization from Intuilab.
 */

/**
 * Inheritance on EventEmitter base class
 * @type {EventEmitter}
 */
MemoryGame.prototype = new EventEmitter();        // Here's where the inheritance occurs
MemoryGame.prototype.constructor = MemoryGame;

/**
 * @constructor
 */
function MemoryGame() {

    var _cardList = [];
    var _attempts = 0;
    var _score = 0;
	var _errorMessage = "";

	var _currentFolder = "Images"; //default folder to load the cards images from.
	var _boardBlocked = false;
	
	var _maxScore = -1;
	
	
	this._lastIndex = 0;
    this._lastAffectedValue = 0;
	
	this._iCardSelected = 0;
	this._card1 = null;
	this._card2 = null;
	
	
	
	
	Object.defineProperty(this, 'CardList',{
        enumerable: true,
		get: function(){ return _cardList; },
        set: function(value){
            if(_cardList != value){
                _cardList = value;                
                this.emit('CardListChanged');
            }
        }
    });	
	Object.defineProperty(this, 'Attempts',{
		enumerable: true,
        get: function(){ return _attempts; },
        set: function(value){
            if(_attempts != value){
                _attempts = value;                
                this.emit('AttemptsChanged');
            }
        }
    });
	Object.defineProperty(this, 'Score',{
		enumerable: true,
        get: function(){ return _score; },
        set: function(value){
            if(_score != value){
                _score = value;                
                this.emit('ScoreChanged');
            }
        }
    });
	Object.defineProperty(this, 'ErrorMessage',{
		enumerable: true,
        get: function(){ return _errorMessage; },
        set: function(value){
            if(_errorMessage != value){
                _errorMessage = value;                
                this.emit('ErrorMessageChanged');
            }
        }
    });
	Object.defineProperty(this, 'CurrentFolder',{
		enumerable: true,
        get: function(){ return _currentFolder; },
        set: function(value){
            if(_currentFolder != value){
                _currentFolder = value;                
                this.emit('CurrentFolderChanged');
            }
        }
    });
	Object.defineProperty(this, 'BoardBlocked',{
		enumerable: true,
        get: function(){ return _boardBlocked; },
        set: function(value){
            if(_boardBlocked != value){
                _boardBlocked = value;                
                this.emit('BoardBlockedChanged');
            }
        }
    });
	Object.defineProperty(this, 'MaxScore',{
		enumerable: true,
        get: function(){ return _maxScore; },
        set: function(value){
            if(_maxScore != value){
                _maxScore = value;                
                this.emit('MaxScoreChanged');
            }
        }
    });
	
	var self = this;
	setTimeout(function() {
         self._loadCards(self.CurrentFolder);         
   }, 500);
	
}

MemoryGame.prototype._loadCards = function (path) {

	if (this.fileService == null)
		this.fileService = intuiface.get('fileService', this);
	
	var self = this;

	this.fileService.getDirectoryContent(path, {
	'success': function(entries)
	{
		for (var index in entries)
		{
			var item = entries[index];
			self._addMedia(item.fullPath);
		}
		self.Shuffle();
		self.emit('CardListChanged', [self.CardList]);
		
		//Save maximum score value
		self.MaxScore = entries.length;
	}, 'error': function (error){
		self.ErrorMessage = error;
		self.emit('ErrorMessageChanged', [self.ErrorMessage]);
	}});
	
}
/**
 * Call this function to add media to the game
 */
MemoryGame.prototype._addMedia = function (mediaURL) {
    var c = new MemoryCard();
    c.MediaURL = mediaURL;
	c.CardId = this._lastIndex++;
    c.CardValue = this._lastAffectedValue;

    //add the memory card twice. They will be shuffled after creation is finished
    this.CardList.push(c);
	
	//add the second memory card, with same Value but different Index
	c = new MemoryCard();
    //c.MediaURL = mediaURL;
	//In this game version, use the version 2 of the card from a different folder. 
	var mediaURL2 = mediaURL.replace("Images", "Images_2");
	c.MediaURL = mediaURL2;
	c.CardId = this._lastIndex++;
    c.CardValue = this._lastAffectedValue++;
    this.CardList.push(c);
};


MemoryGame.prototype.SelectCard = function (_cardId) {
	//if 2 cards are already selected, don't take other ones into account.
	if (this._iCardSelected == 2)
	{
		return;
	}
	for (var index in this.CardList)
	{
		item = this.CardList[index];
		if (item.CardId == _cardId)
		{
			if (this._iCardSelected == 0)
			{
				this._card1 = item;				
			}else 
			{
				this._card2 = item;
			}				
			item.IsSelected = true;
			this._iCardSelected++;
			break;
		}			
	}
	
	
	//2 cards are flipped, compare them.
	if (this._iCardSelected == 2)
	{		
		this._compareSelection();	
	}
		
}


MemoryGame.prototype._compareSelection = function () {
	this.BoardBlocked = true;
	
	console.log("INCREASE ATTEMPTS");
	this.Attempts++;	
	
	if (this._card1.CardValue == this._card2.CardValue)
	{
		console.log("COMPARISON OK FOR ID: "+this._card1.CardValue);
		this.emit("CardsMatch");
		this._card1.IsEnabled = false;
		this._card2.IsEnabled = false;		
		this.Score++;	

		this._card1 = null;
		this._card2 = null;
		this._iCardSelected = 0;
		
		console.log("Score : "+ this.Score + " -- maxScore : " + this.MaxScore);
		if (this.Score == this.MaxScore)
		{
			this._handleSuccess();
		}
		else
		{
			console.log("BLOCK -> FALSE");
			this.BoardBlocked = false;
		}		
	}
	else
	{
		console.log("COMPARISON FAILED");
		this.emit("CardsDoNotMatch");
		var self = this;
		setTimeout(function() {
			
			self._card1.IsSelected = false;
			self._card2.IsSelected = false;		
			
			self._card1 = null;
			self._card2 = null;
			self._iCardSelected = 0;
			
			self.BoardBlocked = false;
	   }, 1000);
		
		
	}

	
	
	
	
	
}



MemoryGame.prototype._handleSuccess = function () {
	this.emit("GameWon");
}

MemoryGame.prototype.Reset = function () {
	for (var index in this.CardList)
	{
		item = this.CardList[index];
		item.IsEnabled = true;
		item.IsSelected = false;
	}
	
	this._card1 = null;
	this._card2 = null;
	this._iCardSelected = 0;
	
	this.Score = 0;
	this.Attempts = 0;
	this.BoardBlocked = false;
	this.Shuffle();
}



/**
 * When creation is done, shuffle the array to display the game cards
 */
MemoryGame.prototype.Shuffle = function () {
    _shuffle(this.CardList);

	this.emit('CardListChanged', [this.CardList]);
	
};

function _shuffle(array) {
  var currentIndex = array.length, temporaryValue, randomIndex;

  // While there remain elements to shuffle...
  while (0 !== currentIndex) {

    // Pick a remaining element...
    randomIndex = Math.floor(Math.random() * currentIndex);
    currentIndex -= 1;

    // And swap it with the current element.
    temporaryValue = array[currentIndex];
    array[currentIndex] = array[randomIndex];
    array[randomIndex] = temporaryValue;
  }

  return array;
}